@@ -5,13 +5,14 @@ from parameterized import parameterized
 from metar_taf_parser.model.enum import Intensity, Phenomenon, Descriptive, WeatherChangeType, CloudQuantity, CloudType, \
     TimeIndicator
 from metar_taf_parser.model.model import AbstractWeatherContainer, Visibility, Wind
-from metar_taf_parser.parser.parser import AbstractParser, MetarParser, _parse_validity, _parse_temperature, TAFParser
+from metar_taf_parser.parser.parser import AbstractParser, MetarParser, _parse_validity, _parse_temperature, TAFParser, \
+    RemarkParser
+from metar_taf_parser.commons.i18n import _
 
 
 class AbstractParserTestCase(unittest.TestCase):
 
     def test_parse_weather_condition(self):
-
         weather_condition = StubParser()._parse_weather_condition('-DZ')
 
         self.assertEqual(Intensity.LIGHT, weather_condition.intensity)
@@ -19,7 +20,7 @@ class AbstractParserTestCase(unittest.TestCase):
         self.assertEqual(1, len(weather_condition.phenomenons))
         self.assertEqual(Phenomenon.DRIZZLE, weather_condition.phenomenons[0])
 
-    def test_parse_weather_condition_mupltiple_phenomenons(self):
+    def test_parse_weather_condition_multiple_phenomenons(self):
         weather_condition = StubParser()._parse_weather_condition('SHRAGR')
 
         self.assertIsNone(weather_condition.intensity)
@@ -27,9 +28,9 @@ class AbstractParserTestCase(unittest.TestCase):
         self.assertListEqual([Phenomenon.RAIN, Phenomenon.HAIL], weather_condition.phenomenons)
 
     def test_tokenize(self):
-
         code = 'METAR KTTN 051853Z 04011KT 1 1/2SM VCTS SN FZFG BKN003 OVC010 M02/M02 A3006 RMK AO2 TSB40 SLP176 P0002 T10171017='
-        expected = ['METAR', 'KTTN', '051853Z', '04011KT', '1 1/2SM', 'VCTS', 'SN', 'FZFG', 'BKN003', 'OVC010', 'M02/M02', 'A3006', 'RMK', 'AO2', 'TSB40', 'SLP176', 'P0002', 'T10171017']
+        expected = ['METAR', 'KTTN', '051853Z', '04011KT', '1 1/2SM', 'VCTS', 'SN', 'FZFG', 'BKN003', 'OVC010',
+                    'M02/M02', 'A3006', 'RMK', 'AO2', 'TSB40', 'SLP176', 'P0002', 'T10171017']
 
         res = StubParser().tokenize(code)
 
@@ -75,7 +76,8 @@ class MetarParserTestCase(unittest.TestCase):
         self.assertEqual('N', metar.runways_info[0].trend)
 
     def test_parse_with_tempo(self):
-        metar = MetarParser().parse('LFBG 081130Z AUTO 23012KT 9999 SCT022 BKN072 BKN090 22/16 Q1011 TEMPO 26015G25KT 3000 TSRA SCT025CB BKN050')
+        metar = MetarParser().parse(
+            'LFBG 081130Z AUTO 23012KT 9999 SCT022 BKN072 BKN090 22/16 Q1011 TEMPO 26015G25KT 3000 TSRA SCT025CB BKN050')
 
         self.assertTrue(metar.auto)
         self.assertEqual(3, len(metar.clouds))
@@ -130,7 +132,6 @@ class MetarParserTestCase(unittest.TestCase):
         self.assertEqual(30, trend.times[0].time.minute)
 
     def test_parse_with_tempo_tl(self):
-
         metar = MetarParser().parse('LFRM 081630Z AUTO 30007KT 260V360 9999 24/15 Q1008 TEMPO FM1700 TL1830 SHRA')
 
         self.assertEqual(1, len(metar.trends))
@@ -191,7 +192,6 @@ class MetarParserTestCase(unittest.TestCase):
         self.assertEqual('> 10km', metar.visibility.distance)
 
     def test_parse_with_cavok(self):
-
         metar = MetarParser().parse('LFPG 212030Z 03003KT CAVOK 09/06 Q1031 NOSIG')
 
         self.assertTrue(metar.cavok)
@@ -202,14 +202,12 @@ class MetarParserTestCase(unittest.TestCase):
         self.assertTrue(metar.nosig)
 
     def test_parse_altimeter_mercury(self):
-
         metar = MetarParser().parse('KTTN 051853Z 04011KT 9999 VCTS SN FZFG BKN003 OVC010 M02/M02 A3006')
 
         self.assertEqual(1017, metar.altimeter)
         self.assertEqual(3, len(metar.weather_conditions))
 
     def test_parse_wind_alternative_form(self):
-
         metar = MetarParser().parse('ENLK 081350Z 26026G40 240V300 9999 VCSH FEW025 BKN030 02/M01 Q0996')
 
         self.assertEqual(260, metar.wind.degrees)
@@ -230,17 +228,17 @@ class MetarParserTestCase(unittest.TestCase):
 
         self.assertEqual('UNAA', metar.station)
         self.assertEqual(340, metar.wind.degrees)
-        self.assertEquals(2, metar.wind.speed)
+        self.assertEqual(2, metar.wind.speed)
         self.assertEqual('MPS', metar.wind.unit)
         self.assertTrue(metar.cavok)
         self.assertTrue(metar.nosig)
-        self.assertEqual('RMK QFE741', metar.remark)
+        self.assertEqual('QFE741', metar.remark)
+        self.assertEqual(1, len(metar.remarks))
 
 
 class FunctionTestCase(unittest.TestCase):
 
     def test_parse_visibility(self):
-
         validity = _parse_validity('3118/0124')
 
         self.assertEqual(31, validity.start_day)
@@ -249,14 +247,12 @@ class FunctionTestCase(unittest.TestCase):
         self.assertEqual(24, validity.end_hour)
 
     def test_parse_temperature_max(self):
-
         temperature = _parse_temperature('TX15/0612Z')
         self.assertEqual(15, temperature.temperature)
         self.assertEqual(6, temperature.day)
         self.assertEqual(12, temperature.hour)
 
     def test_parse_temperature_min(self):
-
         temperature = _parse_temperature('TNM02/0612Z')
 
         self.assertEqual(-2, temperature.temperature)
@@ -264,7 +260,7 @@ class FunctionTestCase(unittest.TestCase):
         self.assertEqual(12, temperature.hour)
 
 
-class TAFPArserTestCase(unittest.TestCase):
+class TAFParserTestCase(unittest.TestCase):
 
     def test_parse_with_invalid_line_breaks(self):
         code = 'TAF LFPG 150500Z 1506/1612 17005KT 6000 SCT012 \n' + 'TEMPO 1506/1509 3000 BR BKN006 PROB40 \n' + 'TEMPO 1506/1508 0400 BCFG BKN002 PROB40 \n' + 'TEMPO 1512/1516 4000 -SHRA FEW030TCU BKN040 \n' + 'BECMG 1520/1522 CAVOK \n' + 'TEMPO 1603/1608 3000 BR BKN006 PROB40 \n TEMPO 1604/1607 0400 BCFG BKN002 TX17/1512Z TN07/1605Z'
@@ -394,8 +390,8 @@ class TAFPArserTestCase(unittest.TestCase):
         self.assertEqual(40, trend_5.probability)
 
     def test_parse_without_line_breaks(self):
-
-        taf = TAFParser().parse('TAF LSZH 292025Z 2921/3103 VRB03KT 9999 FEW020 BKN080 TX20/3014Z TN06/3003Z PROB30 TEMPO 2921/2923 SHRA BECMG 3001/3004 4000 MIFG NSC PROB40 3003/3007 1500 BCFG SCT004 PROB30 3004/3007 0800 FG VV003 BECMG 3006/3009 9999 FEW030 PROB40 TEMPO 3012/3017 30008KT')
+        taf = TAFParser().parse(
+            'TAF LSZH 292025Z 2921/3103 VRB03KT 9999 FEW020 BKN080 TX20/3014Z TN06/3003Z PROB30 TEMPO 2921/2923 SHRA BECMG 3001/3004 4000 MIFG NSC PROB40 3003/3007 1500 BCFG SCT004 PROB30 3004/3007 0800 FG VV003 BECMG 3006/3009 9999 FEW030 PROB40 TEMPO 3012/3017 30008KT')
 
         # Check on time delivery.
         self.assertEqual(29, taf.day)
@@ -519,8 +515,8 @@ class TAFPArserTestCase(unittest.TestCase):
         self.assertEqual(40, tempo1.probability)
 
     def test_parse_without_line_breaks_and_ending_temperature(self):
-
-        taf = TAFParser().parse('TAF KLSV 120700Z 1207/1313 VRB06KT 9999 SCT250 QNH2992INS BECMG 1217/1218 10010G15KT 9999 SCT250 QNH2980INS BECMG 1303/1304 VRB06KT 9999 FEW250 QNH2979INS TX42/1223Z TN24/1213Z')
+        taf = TAFParser().parse(
+            'TAF KLSV 120700Z 1207/1313 VRB06KT 9999 SCT250 QNH2992INS BECMG 1217/1218 10010G15KT 9999 SCT250 QNH2980INS BECMG 1303/1304 VRB06KT 9999 FEW250 QNH2979INS TX42/1223Z TN24/1213Z')
 
         # Check on time delivery.
         self.assertEqual(12, taf.day)
@@ -621,7 +617,8 @@ class TAFPArserTestCase(unittest.TestCase):
         self.assertEqual(55, fm.wind_shear.speed)
 
     def test_parse_with_nautical_miles_visibility(self):
-        taf = TAFParser().parse('TAF AMD CZBF 300939Z 3010/3022 VRB03KT 6SM -SN OVC015 TEMPO 3010/3012 11/2SM -SN OVC009 \nFM301200 10008KT 2SM -SN OVC010 TEMPO 3012/3022 3/4SM -SN VV007 RMK FCST BASED ON AUTO OBS. NXT FCST BY 301400Z')
+        taf = TAFParser().parse(
+            'TAF AMD CZBF 300939Z 3010/3022 VRB03KT 6SM -SN OVC015 TEMPO 3010/3012 11/2SM -SN OVC009 \nFM301200 10008KT 2SM -SN OVC010 TEMPO 3012/3022 3/4SM -SN VV007 RMK FCST BASED ON AUTO OBS. NXT FCST BY 301400Z')
 
         # THEN the visibility of the main event is 6 SM
         self.assertEqual("6SM", taf.visibility.distance)
@@ -634,18 +631,257 @@ class TAFPArserTestCase(unittest.TestCase):
         self.assertTrue(taf.amendment)
 
     def test_parse_with_remark(self):
-        taf = TAFParser().parse('TAF CZBF 300939Z 3010/3022 VRB03KT 6SM -SN OVC015 RMK FCST BASED ON AUTO OBS. NXT FCST BY 301400Z\n TEMPO 3010/3012 11/2SM -SN OVC009 FM301200 10008KT 2SM -SN OVC010 \nTEMPO 3012/3022 3/4SM -SN VV007')
+        taf = TAFParser().parse(
+            'TAF CZBF 300939Z 3010/3022 VRB03KT 6SM -SN OVC015 RMK FCST BASED ON AUTO OBS. NXT FCST BY 301400Z\n TEMPO 3010/3012 11/2SM -SN OVC009 FM301200 10008KT 2SM -SN OVC010 \nTEMPO 3012/3022 3/4SM -SN VV007')
 
         self.assertIsNotNone(taf)
         self.assertIsNotNone(taf.remark)
-        self.assertEqual('RMK FCST BASED ON AUTO OBS. NXT FCST BY 301400Z', taf.remark)
+        self.assertEqual(9, len(taf.remarks))
 
     def test_parse_with_trend_remark(self):
-        taf = TAFParser().parse('TAF CZBF 300939Z 3010/3022 VRB03KT 6SM -SN OVC015\n TEMPO 3010/3012 11/2SM -SN OVC009 FM301200 10008KT 2SM -SN OVC010 TEMPO 3012/3022 3/4SM -SN VV007 RMK FCST BASED ON AUTO OBS. NXT FCST BY 301400Z')
+        taf = TAFParser().parse(
+            'TAF CZBF 300939Z 3010/3022 VRB03KT 6SM -SN OVC015\n TEMPO 3010/3012 11/2SM -SN OVC009 FM301200 10008KT 2SM -SN OVC010 TEMPO 3012/3022 3/4SM -SN VV007 RMK FCST BASED ON AUTO OBS. NXT FCST BY 301400Z')
 
         self.assertEqual(3, len(taf.trends))
         self.assertIsNotNone(taf.trends[2].remark)
-        self.assertEqual('RMK FCST BASED ON AUTO OBS. NXT FCST BY 301400Z', taf.trends[2].remark)
+        self.assertEqual(9, len(taf.trends[2].remarks))
+
+
+class RemarkParserTestCase(unittest.TestCase):
+
+    def test_parse_AO1(self):
+        remarks = RemarkParser().parse('Token AO1 End of remark')
+
+        self.assertEqual(5, len(remarks))
+        self.assertEqual(_('Remark.AO1'), remarks[1])
+
+    def test_parse_AO2(self):
+        remarks = RemarkParser().parse('Token AO2 End of remark')
+
+        self.assertEqual(5, len(remarks))
+        self.assertEqual(_('Remark.AO2'), remarks[1])
+
+    def test_parse_peak_wind_hour(self):
+        remarks = RemarkParser().parse('AO1 PK WND 28045/15')
+
+        self.assertEqual(2, len(remarks))
+        self.assertEqual(_('Remark.AO1'), remarks[0])
+        self.assertEqual(_('Remark.PeakWind').format('280', '45', '', '15'), remarks[1])
+
+    def test_parse_peak_wind_another_hour(self):
+        remarks = RemarkParser().parse('AO1 PK WND 28045/1515')
+
+        self.assertEqual(2, len(remarks))
+        self.assertEqual(_('Remark.AO1'), remarks[0])
+        self.assertEqual(_('Remark.PeakWind').format('280', '45', '15', '15'), remarks[1])
+
+    def test_parse_wind_shift_hour(self):
+        remarks = RemarkParser().parse('AO1 WSHFT 30')
+
+        self.assertEqual(2, len(remarks))
+        self.assertEqual(_('Remark.WindShift').format('', 30), remarks[1])
+
+    def test_parse_wind_shift(self):
+        remarks = RemarkParser().parse('AO1 WSHFT 1530')
+
+        self.assertEqual(2, len(remarks))
+        self.assertEqual(_('Remark.WindShift').format(15, 30), remarks[1])
+
+    def test_parse_wind_shift_frontal(self):
+        remarks = RemarkParser().parse('AO1 WSHFT 1530 FROPA')
+
+        self.assertEqual(2, len(remarks))
+        self.assertEqual(_('Remark.WindShift.FROPA').format(15, 30), remarks[1])
+
+    def test_parse_wind_shift_frontal_at_hour(self):
+        remarks = RemarkParser().parse('AO1 WSHFT 30 FROPA')
+
+        self.assertEqual(2, len(remarks))
+        self.assertEqual(_('Remark.WindShift.FROPA').format('', 30), remarks[1])
+
+    def test_parse_tower_visibility(self):
+        remarks = RemarkParser().parse('AO1 TWR VIS 16 1/2')
+
+        self.assertEqual(2, len(remarks))
+        self.assertEqual(_('Remark.Tower.Visibility').format('16 1/2'), remarks[1])
+
+    def test_parse_surface_visibility(self):
+        remarks = RemarkParser().parse('AO1 SFC VIS 16 1/2')
+
+        self.assertEqual(2, len(remarks))
+        self.assertEqual(_('Remark.Surface.Visibility').format('16 1/2'), remarks[1])
+
+    def test_parse_prevailing_visibility(self):
+        remarks = RemarkParser().parse('AO1 VIS 1/2V2')
+
+        self.assertEqual(2, len(remarks))
+        self.assertEqual(_('Remark.Variable.Prevailing.Visibility').format('1/2', '2'), remarks[1])
+
+    def test_parse_sector_visibility(self):
+        remarks = RemarkParser().parse('AO1 VIS NE 2 1/2')
+
+        self.assertEqual(2, len(remarks))
+        self.assertEqual(_('Remark.Sector.Visibility').format(_('Converter.NE'), '2 1/2'), remarks[1])
+
+    def test_parse_second_location_visibility(self):
+        remarks = RemarkParser().parse('AO1 VIS 2 1/2 RWY11')
+
+        self.assertEqual(2, len(remarks))
+        self.assertEqual(_('Remark.Second.Location.Visibility').format('2 1/2', 'RWY11'), remarks[1])
+
+    def test_parse_tornadic_activity_tornado(self):
+        remarks = RemarkParser().parse('AO1 TORNADO B13 6 NE')
+
+        self.assertEqual(2, len(remarks))
+        self.assertEqual(
+            _('Remark.Tornadic.Activity.Beginning').format(_('Remark.TORNADO'), '', 13, 6, _('Converter.NE')),
+            remarks[1])
+
+    def test_parse_tornadic_activity_tornado_hour(self):
+        remarks = RemarkParser().parse('AO1 TORNADO B1513 6 NE')
+
+        self.assertEqual(2, len(remarks))
+        self.assertEqual(
+            _('Remark.Tornadic.Activity.Beginning').format(_('Remark.TORNADO'), 15, 13, 6, _('Converter.NE')),
+            remarks[1])
+
+    def test_parse_tornadic_activity_funnel_cloud(self):
+        remarks = RemarkParser().parse('AO1 FUNNEL CLOUD B1513E1630 6 NE')
+
+        self.assertEqual(2, len(remarks))
+        self.assertEqual(
+            _('Remark.Tornadic.Activity.BegEnd').format(_('Remark.FUNNELCLOUD'), 15, 13, 16, 30, 6, _('Converter.NE')),
+            remarks[1])
+
+    def test_parse_tornadic_activity_water_sprout_ending_time_minutes(self):
+        remarks = RemarkParser().parse('AO1 WATERSPOUT E16 12 NE')
+
+        self.assertEqual(2, len(remarks))
+        self.assertEqual(
+            _('Remark.Tornadic.Activity.Ending').format(_('Remark.WATERSPOUT'), '', 16, 12, _('Converter.NE')),
+            remarks[1])
+
+    def test_parse_tornadic_activity_watersprout_ending_time(self):
+        remarks = RemarkParser().parse('AO1 WATERSPOUT E1516 12 NE')
+
+        self.assertEqual(2, len(remarks))
+        self.assertEqual(
+            _('Remark.Tornadic.Activity.Ending').format(_('Remark.WATERSPOUT'), 15, 16, 12, _('Converter.NE')),
+            remarks[1])
+
+    def test_parse_precipitation_start_end(self):
+        remarks = RemarkParser().parse('AO1 RAB05E30SNB1520E1655')
+
+        self.assertEqual(3, len(remarks))
+        self.assertEqual(_('Remark.Precipitation.Beg.End').format('', _('Phenomenon.RA'), '', '05', '', 30), remarks[1])
+        self.assertEqual(_('Remark.Precipitation.Beg.End').format('', _('Phenomenon.SN'), 15, 20, 16, 55), remarks[2])
+
+    def test_parse_precipitation_start_end_descriptive(self):
+        remarks = RemarkParser().parse('AO1 SHRAB05E30SHSNB20E55')
+
+        self.assertEqual(3, len(remarks))
+        self.assertEqual(
+            _('Remark.Precipitation.Beg.End').format(_('Descriptive.SH'), _('Phenomenon.RA'), '', '05', '', 30),
+            remarks[1])
+        self.assertEqual(
+            _('Remark.Precipitation.Beg.End').format(_('Descriptive.SH'), _('Phenomenon.SN'), '', 20, '', 55),
+            remarks[2])
+
+    def test_parse_thunderstorm_start(self):
+        remarks = RemarkParser().parse('AO1 TSB0159E30')
+
+        self.assertEqual(2, len(remarks))
+        self.assertEqual(_('Remark.Precipitation.Beg.End').format('', _('Phenomenon.TS'), '01', 59, '', 30), remarks[1])
+
+    def test_parse_thunderstorm_location(self):
+        remarks = RemarkParser().parse('AO1 TS SE')
+
+        self.assertEqual(2, len(remarks))
+        self.assertEqual(_('Remark.Thunderstorm.Location').format(_('Converter.SE')), remarks[1])
+
+    def test_parse_thunderstorm_location_moving(self):
+        remarks = RemarkParser().parse('AO1 TS SE MOV NE')
+
+        self.assertEqual(2, len(remarks))
+        self.assertEqual(_('Remark.Thunderstorm.Location.Moving').format(_('Converter.SE'), _('Converter.NE')),
+                         remarks[1])
+
+    def test_parse_hail_size(self):
+        remarks = RemarkParser().parse('AO1 GR 1 3/4')
+
+        self.assertEqual(2, len(remarks))
+        self.assertEqual(_('Remark.Hail').format('1 3/4'), remarks[1])
+
+    def test_parse_hail_size_less_than(self):
+        remarks = RemarkParser().parse('AO1 GR LESS THAN 1/4')
+
+        self.assertEqual(2, len(remarks))
+        self.assertEqual(_('Remark.Hail.LesserThan').format('1/4'), remarks[1])
+
+    def test_parse_snow_pellets(self):
+        remarks = RemarkParser().parse('AO1 GS MOD')
+
+        self.assertEqual(2, len(remarks))
+        self.assertEqual(_('Remark.Snow.Pellets').format(_('Remark.MOD')), remarks[1])
+
+    def test_parse_virga_direction(self):
+        remarks = RemarkParser().parse('AO1 VIRGA SW')
+
+        self.assertEqual(2, len(remarks))
+        self.assertEqual(_('Remark.Virga.Direction').format(_('Converter.SW')), remarks[1])
+
+    def test_parse_virga(self):
+        remarks = RemarkParser().parse('AO1 VIRGA')
+
+        self.assertEqual(2, len(remarks))
+        self.assertEqual(_('Remark.VIRGA'), remarks[1])
+
+    def test_parse_ceiling_height(self):
+        remarks = RemarkParser().parse('AO1 CIG 005V010')
+
+        self.assertEqual(2, len(remarks))
+        self.assertEqual(_('Remark.Ceiling.Height').format(500, 1000), remarks[1])
+
+    def test_parse_obscurations(self):
+        remarks = RemarkParser().parse('AO1 FU BKN020')
+
+        self.assertEqual(2, len(remarks))
+        self.assertEqual(_('Remark.Obscuration').format(_('CloudQuantity.BKN'), 2000, _('Phenomenon.FU')), remarks[1])
+
+    def test_parse_variable_sky_condition_without_layer(self):
+        remarks = RemarkParser().parse('BKN V OVC')
+
+        self.assertEqual(_('Remark.Variable.Sky.Condition').format(_('CloudQuantity.BKN'), _('CloudQuantity.OVC')),
+                         remarks[0])
+
+    def test_parse_variable_sky_conditions(self):
+        remarks = RemarkParser().parse('BKN014 V OVC')
+        self.assertEqual(
+            _('Remark.Variable.Sky.Condition.Height').format(1400, _('CloudQuantity.BKN'), _('CloudQuantity.OVC')),
+            remarks[0])
+
+    def test_parse_ceiling_second_location(self):
+        remarks = RemarkParser().parse('CIG 002 RWY11')
+
+        self.assertEqual(_('Remark.Ceiling.Second.Location').format(200, 'RWY11'), remarks[0])
+
+    def test_parse_sea_level_pressure(self):
+        remarks = RemarkParser().parse('AO1 SLP134')
+
+        self.assertEqual(_('Remark.Sea.Level.Pressure').format('1013.4'), remarks[1])
+
+    def test_parse_sea_level_pressure_lower(self):
+        remarks = RemarkParser().parse('AO1 SLP982')
+
+        self.assertEqual(_('Remark.Sea.Level.Pressure').format('998.2'), remarks[1])
+
+    def test_parse_snow_increasing_rapidly(self):
+        remarks = RemarkParser().parse('AO1 SNINCR 2/10')
+        self.assertEqual(_('Remark.Snow.Increasing.Rapidly').format(2, 10), remarks[1])
+
+    def test_parse_rmk_slp(self):
+        remarks = RemarkParser().parse('CF1AC8 CF TR SLP091 DENSITY ALT 200FT')
+        self.assertEqual(_('Remark.Sea.Level.Pressure').format('1009.1'), remarks[3])
 
 
 class StubParser(AbstractParser):
